from slai.modules.parameters import from_config
from slai.modules.runtime import detect_runtime, detect_credentials
from slai.clients.model import get_model_client

from requests.auth import HTTPBasicAuth
from importlib import import_module

import requests

REQUESTS_TIMEOUT = 15


def get_inference_client(
    *, model_name, project_name, version, client_id=None, client_secret=None
):
    import_path = from_config(
        "MODEL_INFERENCE_CLIENT",
        "slai.clients.inference.ModelInferenceClient",
    )
    class_ = import_path.split(".")[-1]
    path = ".".join(import_path.split(".")[:-1])

    return getattr(import_module(path), class_)(
        model_name=model_name,
        project_name=project_name,
        version=version,
        client_id=client_id,
        client_secret=client_secret,
    )


class ModelInferenceClient:
    BASE_URL = from_config(
        key="BASE_URL",
        default="https://6zacu5yc29.execute-api.us-east-1.amazonaws.com/development",
    )

    def __init__(
        self,
        *,
        model_name,
        project_name,
        version="latest",
        client_id=None,
        client_secret=None,
    ):
        self.runtime = detect_runtime()

        if client_id is None or client_secret is None:
            credentials = detect_credentials(runtime=self.runtime)
            self.client_id = credentials["client_id"]
            self.client_secret = credentials["client_secret"]
        else:
            self.client_id = client_id
            self.client_secret = client_secret

        self.model_name = model_name
        self.project_name = project_name
        self.version = version

        self._load_model()

    def _load_model(self):
        self.model_client = get_model_client(
            model_name=self.model_name,
            project_name=self.project_name,
            client_id=self.client_id,
            client_secret=self.client_secret,
        )
        self.model = self.model_client.get_model()

    def call(self, payload):
        body = {
            "model_id": self.model["id"],
            "model_version_id": self.version,
            "payload": payload,
        }

        res = requests.post(
            f"{self.BASE_URL}/model/call",
            auth=HTTPBasicAuth(self.client_id, self.client_secret),
            json=body,
            timeout=REQUESTS_TIMEOUT,
        )
        res.raise_for_status()
        return res.json()

    def info(self):
        pass



import boto3
import yaml
import os

from slai.exceptions import ModelNotFound, ProjectNotFound
from slai.clients.cli import SlaiCliClient
from slai.modules.parameters import from_config
from slai.modules.runtime import detect_runtime, detect_credentials, ValidRuntimes

from pathlib import Path
from importlib import import_module


def get_project_client(*, project_name, client_id=None, client_secret=None):
    import_path = from_config(
        "PROJECT_CLIENT",
        "slai.clients.project.ProjectClient",
    )
    class_ = import_path.split(".")[-1]
    path = ".".join(import_path.split(".")[:-1])

    return getattr(import_module(path), class_)(
        project_name=project_name, client_id=client_id, client_secret=client_secret
    )


class ProjectClient:
    def __init__(self, *, project_name=None, client_id=None, client_secret=None):
        self.runtime = detect_runtime()

        self.project_name = project_name
        if client_id is None and client_secret is None:
            self.credentials = detect_credentials(runtime=self.runtime)
        else:
            self.credentials = {"client_id": client_id, "client_secret": client_secret}

        self.cli_client = SlaiCliClient(
            client_id=self.credentials["client_id"],
            client_secret=self.credentials["client_secret"],
        )

    def list_models(self):
        """List models in a slai project."""
        project = self.get_project()
        return project["models"]

    def _load_config_from_disk(self):
        cwd = Path.cwd()
        config = None

        local_config_path = f"{cwd}/.slai/config.yml"

        with open(local_config_path, "r") as f_in:
            try:
                config = yaml.safe_load(f_in)
            except yaml.YAMLError:
                pass

        return config

    def get_credentials(self):
        return self.credentials

    def get_project_name(self):
        """Retrieve project configuration values."""
        if self.project_name is None:
            config = self._load_config_from_disk()
            return config["project_name"]

        return self.project_name

    def get_project(self):
        """Retrieve project configuration values."""
        project = self.cli_client.retrieve_project(project_name=self.get_project_name())

        return project


class MockProjectClient:
    def __init__(self):
        self.runtime = detect_runtime()
        self.project_id = "MOCK_PROJECT_ID"

    def list_models(self):
        project = {"models": []}
        return project["models"]

    def get_project_name(self):
        return "MOCK_PROJECT_NAME"

    def get_project(self):
        project = {
            "id": self.project_id,
            "name": "some_project",
            "project_folder_google_drive_file_id": None,
            "created": "2021-02-21T11:40:06.607882",
            "updated": "2021-02-21T11:40:06.653309",
            "models": [],
        }
        return project



from slai.modules.parameters import from_config
from slai.modules.runtime import detect_runtime, detect_credentials, ValidRuntimes
from slai.clients.inference import get_inference_client


class Model:
    def __init__(
        self,
        *,
        model_name,
        project_name,
        version="latest",
        client_id=None,
        client_secret=None
    ):
        self.inference_client = get_inference_client(
            model_name=model_name,
            project_name=project_name,
            version=version,
            client_id=client_id,
            client_secret=client_secret,
        )

    def __call__(self, **inputs):
        return self.inference_client.call(payload=inputs)

    def info(self):
        pass



import requests

from requests.auth import HTTPBasicAuth
from importlib import import_module

from slai.modules.parameters import from_config

REQUESTS_TIMEOUT = 15


def get_identity_client(*, client_id=None, client_secret=None):
    import_path = from_config(
        "IDENTITY_CLIENT",
        "slai.clients.identity.IdentityClient",
    )
    class_ = import_path.split(".")[-1]
    path = ".".join(import_path.split(".")[:-1])
    return getattr(import_module(path), class_)(
        client_id=client_id, client_secret=client_secret
    )


class IdentityClient:
    BASE_URL = from_config(
        key="BASE_URL",
        default="https://6zacu5yc29.execute-api.us-east-1.amazonaws.com/development",
    )

    def __init__(self, *, client_id, client_secret):
        self.client_id = client_id
        self.client_secret = client_secret

    def validate_notebook_auth_token(self, *, token):
        body = {"action": "retrieve", "token": token}

        res = requests.post(
            f"{self.BASE_URL}/app/notebook-auth",
            auth=None,
            headers={},
            json=body,
            timeout=REQUESTS_TIMEOUT,
        )
        res.raise_for_status()
        return res.json()

    def get_user(self):
        body = {"action": "retrieve"}

        res = requests.post(
            f"{self.BASE_URL}/cli/user",
            auth=HTTPBasicAuth(self.client_id, self.client_secret),
            headers={},
            json=body,
            timeout=REQUESTS_TIMEOUT,
        )
        res.raise_for_status()
        return res.json()



import yaml
import io

from pathlib import Path

from slai.exceptions import ModelNotFound
from slai.clients.s3 import get_s3_client
from slai.clients.gdrive import get_google_drive_client
from slai.clients.project import get_project_client
from slai.clients.cli import get_cli_client
from slai.modules.parameters import from_config
from slai.modules.runtime import detect_runtime
from importlib import import_module


def get_model_client(*, model_name, project_name, client_id=None, client_secret=None):
    import_path = from_config(
        "MODEL_CLIENT",
        "slai.clients.model.ModelClient",
    )
    class_ = import_path.split(".")[-1]
    path = ".".join(import_path.split(".")[:-1])
    return getattr(import_module(path), class_)(
        model_name=model_name,
        project_name=project_name,
        client_id=client_id,
        client_secret=client_secret,
    )


class ModelClient:
    def __init__(
        self, *, model_name=None, project_name=None, client_id=None, client_secret=None
    ):
        self.runtime = detect_runtime()
        if self.runtime == "local":
            try:
                # we only need the google client when we're running locally
                self.gdrive_client = get_google_drive_client()
            except FileNotFoundError:
                pass

        self.project_client = get_project_client(
            project_name=project_name, client_id=client_id, client_secret=client_secret
        )

        if client_id is None and client_secret is None:
            self.credentials = self.project_client.get_credentials()
        else:
            self.credentials = {"client_id": client_id, "client_secret": client_secret}

        self.api_client = get_cli_client(
            client_id=self.credentials["client_id"],
            client_secret=self.credentials["client_secret"],
        )

        self.model_name = model_name
        self.model = self.get_model()

    def get_project_name(self):
        return self.project_client.get_project_name()

    def create_model_folder(self, *, project_google_drive_folder_id):
        model_google_drive_folder_id = self.gdrive_client.create_folder(
            name=f"{self.model['name']}",
            parent_ids=[project_google_drive_folder_id],
        )
        return model_google_drive_folder_id

    def create_model_artifact(
        self, *, model_data, artifact_type, artifact_requirements, model_version_id=None
    ):
        if model_version_id is None:
            model_version_id = self.model["model_version_id"]

        model_artifact = self.api_client.create_model_artifact(
            model_version_id=model_version_id,
            model_data=model_data,
            artifact_type=artifact_type,
            artifact_requirements=artifact_requirements,
        )
        return model_artifact

    def create_model_deployment(
        self,
        *,
        model_artifact_id,
        model_handler_data,
        requirements,
    ):
        model_deployment = self.api_client.create_model_deployment(
            model_artifact_id=model_artifact_id,
            model_handler_data=model_handler_data,
            requirements=requirements,
        )
        return model_deployment

    def upload_model_notebook(
        self, *, model_google_drive_folder_id, notebook_path, file_id=None
    ):
        model_data = self.get_model()

        model_notebook_google_drive_file_id = self.gdrive_client.upload_file(
            filename=f"{model_data['name']}.ipynb",
            local_path=notebook_path,
            parent_ids=[
                model_google_drive_folder_id,
            ],
            file_id=file_id,
        )

        return model_notebook_google_drive_file_id

    def update_model(self, *, key, value):
        model_data = self.get_model()
        model_data[key] = value
        model_data = self.api_client.update_model(model_data=model_data)
        return model_data

    def get_model(self):
        model_data = self.api_client.retrieve_model(
            project_name=self.project_client.get_project_name(),
            name=self.model_name,
        )
        return model_data

    def get_latest_model_artifact(self, model_version_id=None):
        model_data = self.get_model()

        if model_version_id is None:
            model_version_id = model_data["model_version_id"]

        model_artifact = self.api_client.retrieve_model_artifact(
            model_version_id=model_version_id,
            model_artifact_id=None,
        )
        return model_artifact

    def download_latest_model_notebook(
        self, local_path, model_notebook_google_drive_file_id
    ):
        self.gdrive_client.download_file(
            file_id=model_notebook_google_drive_file_id,
            local_path=local_path,
        )


class MockModelClient:
    def __init__(self, *, model_name, project_name):
        pass

    def get_project_name(self):
        return "MOCK_PROJECT_NAME"

    def create_model_folder(self):
        return

    def create_model_artifact(
        self, *, model_data, artifact_type, artifact_requirements
    ):
        return {
            "id": "60329220dd191a68cfb474ec",
            "model_version_id": "60328ce6dd191a6608bb7da9",
            "artifact_type": artifact_type,
            "artifact_requirements": {"torch": "==1.70.0a0"},
            "created": "2021-02-21T12:02:24.503850",
        }

    def upload_template_model_notebook(self, template_notebook_path):
        return

    def update_model(self, *, key, value):
        model_data = {
            "id": "60329220dd191a68cfb474ec",
            "project_name": "something",
            "project_id": "60328ce6dd191a6608bb7da9",
            "name": "something1",
            "model_s3_data_bucket_name": None,
            "notebook_uri": None,
            "model_version_id": "60329220dd191a68cfb474ed",
            "created": "2021-02-21T12:02:24.503850",
            "updated": "2021-02-21T12:02:24.635203",
        }
        model_data[key] = value
        return model_data

    def get_model(self):
        return {
            "id": "60329220dd191a68cfb474ec",
            "project_name": "something",
            "project_id": "60328ce6dd191a6608bb7da9",
            "name": "something1",
            "model_s3_data_bucket_name": None,
            "notebook_uri": None,
            "model_version_id": "60329220dd191a68cfb474ed",
            "created": "2021-02-21T12:02:24.503850",
            "updated": "2021-02-21T12:02:24.635203",
        }

    def download_latest_model_notebook(self, local_path):
        return



import os
import re
import importlib
import sys

from slai.dataset import DataSet
from slai.clients.model import get_model_client
from slai.exceptions import UnsupportedModelFormat, UnableToConvertNotebook
from slai.modules.model_saver import ModelSaver, ValidModelFrameworks
from slai.modules.runtime import detect_runtime, ValidRuntimes


class ModelVersion:
    def __init__(self, *, model_name, project_name, model_version_id=None):
        self.runtime = detect_runtime()

        self.model_name = model_name
        self.project_name = project_name

        self.model_client = get_model_client(
            model_name=self.model_name, project_name=self.project_name
        )
        self.project = self.model_client.project_client.get_project()

        self.model = self.model_client.get_model()

        if model_version_id is None:
            self.model_version_id = self.get_version()
        else:
            self.model_version_id = model_version_id

    def get_version(self):
        if self.model_version_id is None:
            self.model = self.model_client.get_model()
            self.model_version_id = self.model["model_version_id"]

        return self.model_version_id

    def dataset(self):
        return DataSet(model=self.model, model_version=self.model_version_id)

    def _get_framework_name(self, model):
        module = None

        if model is not None:
            _model_class = repr(model.__class__.__bases__[0])
            _matches = re.findall(r"\<class\s\'([a-zA-Z0-9\.]+)\'\>", _model_class)

            if len(_matches) >= 1:
                module = _matches[0].split(".")[0].upper()

        return module

    def _handle_collab_trainer(self):
        _subprocess = importlib.import_module("subprocess")

        trainer_path = f"/content/drive/MyDrive/slai-{self.project_name}-{self.project['id']}/{self.model_name}/{self.model_name}.py"  # noqa
        notebook_path = f"/content/drive/MyDrive/slai-{self.project_name}-{self.project['id']}/{self.model_name}/{self.model_name}.ipynb"  # noqa

        return_code = _subprocess.call(
            f"jupyter nbconvert {notebook_path} --to=python", shell=True
        )
        if return_code != 0:
            raise UnableToConvertNotebook("could_not_convert_notebook")

        spec = importlib.util.spec_from_file_location("trainer", trainer_path)
        trainer_module = importlib.util.module_from_spec(spec)

        sys.modules["trainer"] = trainer_module
        sys.modules["__main__"] = trainer_module

    def save(
        self,
        *,
        model,
    ):
        """
        Save a model artifact to the slai backend
        """
        framework_name = self._get_framework_name(model)
        model_data = None

        if self.runtime == ValidRuntimes.Collab:
            self._handle_collab_trainer()

        # torch
        if framework_name == ValidModelFrameworks.Torch:
            model_data, requirements = ModelSaver.save_pytorch(model)
        elif framework_name in []:
            pass

        if model_data is None:
            raise UnsupportedModelFormat("invalid_model_format")

        model_artifact = self.model_client.create_model_artifact(
            model_data=model_data,
            artifact_type=framework_name,
            artifact_requirements=requirements,
            model_version_id=self.model_version_id,
        )
        return model_artifact
